package it.uniroma1.dis.mfis.redia.verifint.model.realization.classDiagrams;

import java.io.Serializable;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.Set;

import it.uniroma1.dis.mfis.redia.verifint.model._dataTypes.ElementId;
import it.uniroma1.dis.mfis.redia.verifint.model._exceptions.AssociationTargetException;
import it.uniroma1.dis.mfis.redia.verifint.model._interfaces.Identifiable;
import it.uniroma1.dis.mfis.redia.verifint.model._interfaces.NameIdentifiable;
import it.uniroma1.dis.mfis.redia.verifint.model.realization.classDiagrams._links.RCDDiagram_isComposedBy_RCDElement;
import it.uniroma1.dis.mfis.redia.verifint.model.constraints._interfaces.XConstrainable;
import it.uniroma1.dis.mfis.redia.verifint.model.constraints._interfaces.XConstraint;
import it.uniroma1.dis.mfis.redia.verifint.model.constraints._links.XConstrainable_satisfies_XConstraint;
import it.uniroma1.dis.mfis.redia.verifint.model._links.*;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams.CCDElement;

public abstract class RCDElement implements Serializable, NameIdentifiable, XConstrainable {
	public static Long id_assigner = 1L;
	public static final String ELEMENT_PREFIX = "DCC";
	public static final String ID_PREFIX = ELEMENT_PREFIX + "_Element_";
	public static final String DEFAULT_EMPTY_NAME = "";
	
	private String name;
	
	private ElementId id;
	private RCDDiagram diagram;
	private SortedSet<XConstraint> constraints;
	private Set<CCDElement> ccdElements;
	
	RCDElement(String name) {
		synchronized (id_assigner) {
			this.setId(RCDElement.id_assigner++);
		}
		this.name = name;
		this.constraints = new TreeSet<XConstraint>();
	}
	
	RCDElement() { this(DEFAULT_EMPTY_NAME); }

	public String getName() { return this.name; }

	public void setName(String name) { this.name = name; }

	public ElementId getId() { return id; }

	private void setId(Long numericSuffix) {
		this.setId(new ElementId(ID_PREFIX + numericSuffix.toString()));
	}

	protected void setId(ElementId id) { this.id = id; }

	public RCDDiagram getDiagram() { return this.diagram; }

	protected void setDiagram(RCDDiagram diagram) { this.diagram = diagram; }

	public void setDiagram(RCDDiagram_isComposedBy_RCDElement link)
	throws AssociationTargetException {
		if (!link.getElement().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.diagram = link.getDiagram();
	}

	public void unsetDiagram(RCDDiagram_isComposedBy_RCDElement link)
	throws AssociationTargetException {
		if (!link.getElement().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.diagram = null;
	}

	public SortedSet<XConstraint> getConstraints() {
		return new TreeSet<XConstraint>(constraints);
	}

	protected void setConstraints(SortedSet<XConstraint> constraints) {
		this.constraints = constraints;
	}
	
	public void addConstraint(XConstrainable_satisfies_XConstraint link)
	throws AssociationTargetException {
		if (!link.getConstrainedElement().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.constraints.add(link.getConstraint());
	}

	public void removeConstraint(XConstrainable_satisfies_XConstraint link)
	throws AssociationTargetException {
		if (!link.getConstrainedElement().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.constraints.remove(link.getConstraint());
	}

	public int compareTo(Identifiable o) {
		return
			this.getId().compareTo(o.getId());
	}

	
	public boolean equals(Object obj) {
		return
			this.getId().equals(((RCDElement)obj).getId());
	}

	
	public String toString() { return this.getIdentifierName(); }

	public String getIdentifierName() {
		return
			this.getDiagram().getName() +
			NameIdentifiable.IDENTIFIER_NAME_SEPARATOR +
			this.getName();
	}
	
	
	public void addCcdElement(CCDElement_projects_RCDElement link)
	throws AssociationTargetException {
		if (!link.getRCDElement().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.ccdElements.add(link.getCCDElement());
	}

	public void removeCcdElement(CCDElement_projects_RCDElement link)
	throws AssociationTargetException {
		if (!link.getRCDElement().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.ccdElements.remove(link.getCCDElement());
	}
	public SortedSet<CCDElement> getCCDElement() {
		return new TreeSet<CCDElement>(ccdElements);
	}
	
}