package it.uniroma1.dis.mfis.redia.verifint.model.realization.utilities;

import it.uniroma1.dis.mfis.redia.verifint.model.analysis.utilities.FormulaFOL;
import it.uniroma1.dis.mfis.redia.verifint.model.realization.realizativeSpecifications.RSOperation;
import it.uniroma1.dis.mfis.redia.verifint.model.realization.classDiagrams.RCDOperation;
import java.io.*;

public class JMLPrecondition extends OperationPrecondition {
	
	private String formulaJml;
	
	public JMLPrecondition(String formulaJml) {
		this.formulaJml = formulaJml;
		savePrecondition();
	}
	
	public String JMLPrecondition() {
		return formulaJml;
	}
	public void setJMLPrecondition(String formulaJml) {
		this.formulaJml = formulaJml;
		savePrecondition();
	}
	
	public static String toJML(FormulaFOL formula) {
		
		String ris = formula.toString();
		ris = OLD(ris);
		ris = EXIST(ris);
		ris = ALL(ris);
		ris = IMPLIES(ris);
		return ris;
	}
	
	private static int cercaSimbolo(String origine,String simbolo) {
		int i = 0; 
				
		while(i<origine.length() && i+simbolo.length()<=origine.length()) {
			if(origine.substring(i,i+simbolo.length()).toUpperCase().equals(simbolo)) return i;
			else i++;
		}
		return -1;
	}
	private static int cercaUltimoAND(String origine) {
		int i=origine.length();
		while(i>=0) {
			if(origine.substring(i-1,i).equals("&")) return i;
			else i--;
		}
		return -1;
	}
	private static int cercaUltimaParentesiChiusa(String origine) {
		int temp = 0;
		for(int i=0;i<origine.length();i++) {
			if(origine.substring(i,i+1).equals(")")) {
				temp+=-1;
				if(temp==0) return i;
			}
			if(origine.substring(i,i+1).equals("(") ) {
				temp++;
			}
		}
		return -1;
		
	}
	private static String ALL(String formula){
		String ris = new String();
		for(int i=0;i<formula.length();i++) {
			boolean controlloFORALL = ((formula.length()-i) >= 3  && formula.substring(i,i+3).toUpperCase().equals("ALL") );
			
			if(controlloFORALL)  {
				
				int da = cercaSimbolo(formula.substring(i,formula.length()),"(")+i+1;
				int a = cercaSimbolo(formula.substring(i,formula.length()),")")+i;
				int daIMPLIES = cercaSimbolo(formula.substring(i,formula.length()),"IMPLIES")+i+8;
				int aIMPLIES = cercaSimbolo(formula.substring(daIMPLIES,formula.length()),")")+daIMPLIES;
				ris+="\\FORALL int "+formula.substring(i+4,i+5)+";"+formula.substring(da,a)+";"+formula.substring(daIMPLIES,aIMPLIES)+")";
				i=aIMPLIES;
				
			}
			if(formula.length()-i-1 >= 0) ris+=formula.substring(i,i+1).toLowerCase();
		}	
		return ris;
		
	}
	private static String EXIST(String formula) {
		String ris = new String();
		for(int i=0;i<formula.length();i++) {
			boolean controlloEXIST = ((formula.length()-i) >= 6 && formula.substring(i,i+6).equalsIgnoreCase("EXISTS") );
			
			if(controlloEXIST)  {
				int da = cercaSimbolo(formula.substring(i,formula.length()),"(")+i+1;
				int a = cercaUltimaParentesiChiusa(formula.substring(i,formula.length()))+i;
				int ultimaCondizione = cercaUltimoAND(formula.substring(da,a))+da;
				
				ris+="(\\EXIST int "+formula.substring(i+7,i+8)+";"+formula.substring(da,ultimaCondizione-2)+";"+formula.substring(ultimaCondizione,a);
				i=a+1;
				
			}
		if(formula.length()-i-1 >= 0) ris+=formula.substring(i,i+1).toLowerCase();
		}
		
		return ris;
		
	}
	private static String OLD(String formula) {
		String ris = new String();
		for(int i=0;i<formula.length();i++) {
			boolean controlloOLD = ((formula.length()-i) >= 3 && formula.substring(i,i+3).toUpperCase().equals("OLD") );
			
			if(controlloOLD)  {
				ris+="\\OLD";
				i=i+3;
			}
		if(formula.length()-i-1 >= 0) ris+=formula.substring(i,i+1).toLowerCase();
		}
		
		return ris;
	}
		

	private static String IMPLIES(String formula) {
		String ris = new String();
		for(int i=0;i<formula.length();i++) {
				
			boolean controlloIMPLIES = ((formula.length()-i) >= 7 && formula.substring(i,i+7).toUpperCase().equals("IMPLIES") );
			
			if(controlloIMPLIES)  {
				ris+="==>";
				i=i+7;
			}
		if(formula.length()-i-1 >= 0) ris+=formula.substring(i,i+1).toLowerCase();
		}
		
		return ris;
	}
	
	private static String cancellaSpazi(String s) {
		String temp = new String(s);
		for(int i=0;i<temp.length();i++) {
			if(!temp.substring(i,i+1).equals(" "))  i++;
			else if(temp.length()-i>0 && temp.substring(i+1,i+2).equals(" ")) {
				temp = temp.substring(0,i) + cancellaSpazi(temp.substring(i+1));
			}
		}
		return temp;
		
	}
	
	private void savePrecondition() {
		
		RSOperation rsOperation = this.getRsOperation();
		RCDOperation rcdDefOperation = rsOperation.getRcdDefOperation();
		String operationName = rcdDefOperation.getName();
		
		try {
		    FileWriter fw =new FileWriter(operationName+".txt");
		    BufferedWriter bw =new BufferedWriter(fw);
    		
    		bw.write(formulaJml+"\n");	 		
			bw.close();
			fw.close();
		}
		catch(FileNotFoundException fnfe) { System.out.println("File not found "+fnfe.getMessage());}
		catch(IOException ioe) { System.out.println("IO Error "+ioe.getMessage()); }			
	}
}